@@ -1,5 +1,5 @@
 /* GPLv2 applies
- * SVN revision: $Revision: 771 $
+ * SVN revision: $Revision: 863 $
  * (C) 2006-2014 by folkert@vanheusden.com
  */
 #define _GNU_SOURCE
@@ -236,13 +236,16 @@ void show_channel_from_list(NEWWIN *win, int vc_index, int y)
 	int server_nr = vc_list -> server_index[vc_index];
 	int channel_nr = vc_list -> channel_index[vc_index];
 
+	server *ps = &server_list[server_nr];
+	channel *pc = &ps -> pchannels[channel_nr];
+
 	if (theme.channellist_newlines_markchar != ' ')
 	{
-		mark = server_list[server_nr].pchannels[channel_nr].new_entry == NONE ? ' ' : theme.channellist_newlines_markchar;
+		mark = pc -> new_entry == NONE ? ' ' : theme.channellist_newlines_markchar;
 
-		if (server_list[server_nr].pchannels[channel_nr].new_entry == YOU)
+		if (pc -> new_entry == YOU)
 			textcolor = highlight_colorpair;
-		else if (server_list[server_nr].pchannels[channel_nr].new_entry == META)
+		else if (pc -> new_entry == META)
 			textcolor = meta_colorpair;
 		else
 			textcolor = 0;
@@ -267,17 +270,17 @@ void show_channel_from_list(NEWWIN *win, int vc_index, int y)
 		/*if (server_list[server_nr].minimized)
 			color_on(win, theme.color_channellist_minimized_server); */
 
-		if (server_list[server_nr].description)
-			limit_print(win, win -> ncols - 0, y, 0, "%c%s", mark, server_list[server_nr].description);
+		if (ps -> description)
+			limit_print(win, win -> ncols - 0, y, 0, "%c%s", mark, ps -> description);
 		else
-			limit_print(win, win -> ncols - 0, y, 0, "%c%s", mark, server_list[server_nr].server_host);
+			limit_print(win, win -> ncols - 0, y, 0, "%c%s", mark, ps -> server_host);
 
 		/*if (server_list[server_nr].minimized)
 			color_off(win, theme.color_channellist_minimized_server);*/
 	}
 	else
 	{
-		limit_print(win, win -> ncols - 1, y, 0, "%c %s", mark, server_list[server_nr].pchannels[channel_nr].channel_name);
+		limit_print(win, win -> ncols - 1, y, 0, "%c %s", mark, pc -> channel_name);
 	}
 
 	if (sc != -1)
@@ -289,7 +292,7 @@ void show_channel_from_list(NEWWIN *win, int vc_index, int y)
 
 void show_channel_list(NEWWIN *channel_window)
 {
-	int channel_index;
+	int channel_index = -1;
 
 	create_visible_channels_list();
 
@@ -734,3 +737,10 @@ void channelwindow_mouse(mmask_t buttons, int x, int y)
 		set_cursor_mode(CM_CHANNELS);
 	}
 }
+
+channel *gch(int sr, int ch)
+{
+	server *ps = &server_list[sr];
+
+	return &ps -> pchannels[ch];
+}
