@@ -1,5 +1,5 @@
 /* GPLv2 applies
- * SVN revision: $Revision: 696 $
+ * SVN revision: $Revision: 810 $
  * (C) 2006-2014 by folkert@vanheusden.com
  */
 typedef enum { DCC_RECEIVE_FILE, DCC_SEND_FILE, DCC_CHAT } dcc_type_t;
@@ -10,7 +10,7 @@ typedef struct
 {
 	dcc_type_t mode;
 	dcc_conn_state_t state;
-	int fd_conn;
+	int fd_conn, ifd;
 	int fd_file;
 	char *filename;
 	int server_nr, channel_nr;
@@ -24,8 +24,8 @@ extern char *dcc_path;
 
 void init_dcc(void);
 void free_dcc(void);
-void init_recv_dcc(const char *filename, unsigned long int ip, int port, int server_index, int channel_index);
-int init_send_dcc(const char *filename, int server_index, const char *nick);
+void init_recv_dcc(const char *filename, const char *addr, int port, int server_index, int channel_index);
+int init_send_dcc(const char *filename, int server_index, int channel_index, const char *nick);
 void set_dcc_state(int index, dcc_conn_state_t state);
 dcc_conn_state_t get_dcc_state(int index);
 int dcc_send(DCC_t *pnt);
