@@ -1,5 +1,5 @@
 /* GPLv2 applies
- * SVN revision: $Revision: 775 $
+ * SVN revision: $Revision: 863 $
  * (C) 2006-2014 by folkert@vanheusden.com
  */
 #define _GNU_SOURCE
@@ -125,6 +125,19 @@ void headline_channel_msg(server *ps, channel *pc, const char *msg)
 	free(headline);
 }
 
+void log_to_user_window(int sr, const char *nick, const char *what)
+{
+	int ch = -1;
+
+	if (create_channel_for_meta_requests)
+		ch = create_channel(sr, nick);
+	else
+		ch = find_channel_index(sr, nick);
+
+	if (ch != -1)
+		log_channel(sr, ch, nick, what, FALSE);
+}
+
 int process_server_do_line(int sr, const char *string)
 {
 	int rc = 0;
@@ -487,6 +500,9 @@ int process_server_do_line(int sr, const char *string)
 
 		free(channels_str_before);
 		free(channels_str_after);
+
+		if (keep_channels_sorted)
+			sort_channels(sr);
 	}
 	else if (strcmp(cmd, "INVITE") == 0)	/* INVITE */
 	{
@@ -757,47 +773,71 @@ int process_server_do_line(int sr, const char *string)
 	else if (cmd_nr == 311)	/* reply to whois */
 	{
 		/* 311     RPL_WHOISUSER
-		   "<nick> <user> <host> * :<real name>" */
-		/* :irc.xs4all.nl 311 mynick hisnick hisuser hishost * :hisname */
+		   "<nick> <user> <host> * :<real name>"
+		   :irc.xs4all.nl 311 mynick hisnick hisuser hishost * :hisname */
+		if (string_array_get_n(&before_parts) >= 2 && pars_after_colon != NULL)
+		{
+			char *temp = NULL;
+			const char *nick = string_array_get(&before_parts, 1);
+			const char *full_name = pars_after_colon;
+			int index = 0;
 
-		/* FIXME */
+			for(index=0; index<ps -> n_channels; index++)
+				change_name(sr, index, nick, full_name);
+
+			if (string_array_get_n(&before_parts) >= 4)
+			{
+				char *user_host = NULL;
+
+				asprintf(&user_host, "%s@%s", string_array_get(&before_parts, 3), string_array_get(&before_parts, 4));
+				update_user_host(sr, user_host);
+
+				free(user_host);
+			}
+
+			asprintf(&temp, "%s is %s", nick, full_name);
+			update_statusline(sr, 0, "%s\n", temp);
+
+			if (string_array_get_n(&before_parts) >= 2)
+				log_to_user_window(sr, string_array_get(&before_parts, 1), temp);
+
+			free(temp);
+		}
+	}
+	else if ((cmd_nr >= 312 && cmd_nr <= 319) || cmd_nr == 330) /* more WHOIS info */
+	{
+		if (string_array_get_n(&before_parts) >= 2)
+		{
+			char *temp = NULL;
+			asprintf(&temp, "%s %s", str_or_nothing(pars_before_colon), str_or_nothing(pars_after_colon));
+			log_to_user_window(sr, string_array_get(&before_parts, 1), temp);
+			free(temp);
+		}
 	}
 	else if (cmd_nr == 321)		/* LIST start */
 	{
-		int index = 0;
+		update_statusline(sr, 0, "Receiving list of channels/topics...");
 
-		for(index=0; index<ps -> n_all_channels; index++)
-			myfree(ps -> all_channels[index]);
-
-		myfree(ps -> all_channels);
-
-		ps -> n_all_channels = 0;
-		ps -> all_channels = NULL;
-
-		ps -> all_channels_complete = FALSE;
+		free_channel_list(sr);
 	}
-	else if (cmd_nr == 322)		/* LIST part */
+	else if (cmd_nr == 322 && string_array_get_n(&before_parts) >= 2)	/* LIST part */
 	{
-		char *space = strchr(pars, ' '); /* find end of some nick, start of channel */
+		const char *channel_str = string_array_get(&before_parts, 1);
+		const char *topic = str_or_nothing(pars_after_colon);
 
-		if (space)
-		{
-			char *ch_start = space + 1;
+		ps -> channel_list = (channel_topic_t *)realloc(ps -> channel_list, sizeof(channel_topic_t) * (ps -> channel_list_n + 1));
+		ps -> channel_list[ps -> channel_list_n].channel = strdup(channel_str);
+		ps -> channel_list[ps -> channel_list_n].topic = strdup(topic);
 
-			space = strchr(ch_start, ' ');
-			if (space)
-			{
-				*space = 0x00;
-
-				ps -> all_channels = (const char **)realloc(ps -> all_channels, sizeof(const char *) * (ps -> n_all_channels + 1));
-				ps -> all_channels[ps -> n_all_channels] = strdup(ch_start);
-				ps -> n_all_channels++;
-			}
-		}
+		ps -> channel_list_n++;
 	}
 	else if (cmd_nr == 323)		/* LIST end */
 	{
-		ps -> all_channels_complete = TRUE;
+		ps -> channel_list_complete = TRUE;
+
+		qsort(ps -> channel_list, ps -> channel_list_n, sizeof(channel_topic_t), compare_channel_list_item);
+
+		update_statusline(sr, 0, "List of channels/topics successfully received");
 	}
 	else if (cmd_nr == 352)		/* reply to 'WHO' */
 	{
