@@ -1,5 +1,5 @@
 /* GPLv2 applies
- * SVN revision: $Revision: 776 $
+ * SVN revision: $Revision: 819 $
  * (C) 2006-2014 by folkert@vanheusden.com
  */
 #define _GNU_SOURCE
@@ -45,6 +45,7 @@
 #include "user.h"
 #include "ctcp.h"
 #include "headlines.h"
+#include "help.h"
 
 NEWWIN *chat_window_border = NULL, *channel_window_border = NULL, *input_window_border = NULL;
 NEWWIN *chat_window = NULL, *input_window = NULL, *channel_window = NULL, *topic_line_window = NULL, *headline_window = NULL;
@@ -415,7 +416,6 @@ int log_channel(int iserver, int channel_nr, const char *user, const char *strin
 	time_t rnow = time(NULL);
 	struct tm *ptm = localtime(&rnow);
 	char *ts = NULL;
-	BOOL is = FALSE;
 	server *ps = &server_list[iserver];
 	channel *pc = &ps -> pchannels[channel_nr];
 	BOOL output_to_terminal = iserver == current_server && channel_nr == current_server_channel_nr;
@@ -433,20 +433,25 @@ int log_channel(int iserver, int channel_nr, const char *user, const char *strin
 		}
 	}
 
-	if (user != NULL && notify_nick && nick_hit(ps -> nickname, string) && strcasecmp(user, ps -> nickname) == 0)
+	if (user != NULL && nick_hit(ps -> nickname, string) && strcasecmp(user, ps -> nickname) != 0)
 	{
-		const char *pars[3] = { NULL };
+		if (notify_nick)
+		{
+			const char *pars[3] = { NULL };
 
-		pars[0] = user;
-		pars[1] = string;
-		pars[2] = NULL;
+			pars[0] = user;
+			pars[1] = string;
+			pars[2] = NULL;
 
-		run(notify_nick, pars);
+			run(notify_nick, pars);
+		}
 
 		add_headline(TRUE, headline);
 	}
-
-	check_headline_matches(string, headline);
+	else
+	{
+		check_headline_matches(string, headline);
+	}
 
 	if (pc -> last_entry)
 	{
@@ -491,7 +496,7 @@ int log_channel(int iserver, int channel_nr, const char *user, const char *strin
 		asprintf(&str_buffer, "%s > %s", ts, string);
 	}
 
-	if (user == NULL || !is_ignored(iserver, channel_nr, user, &is) || is == FALSE)
+	if (user == NULL || is_ignored(iserver, channel_nr, user) == FALSE)
 	{
 		time_t last_entry = pc -> last_entry;
 
@@ -964,11 +969,11 @@ void update_channel_border(int server_index)
 	if (chat_window_border && server_index == current_server)
 	{
 		const char f1_help[] = "[ Press F1 for help ]";
-		const char key_right_select[] = "[ Press right cursor key to select channel ]";
+		const char *tooltip = get_tooltip();
 		int f1_x = 0, f1_y = 0;
 		int kr_x = 0, kr_y = -1;
 		int f1_help_len = sizeof f1_help;
-		int key_right_select_len = sizeof key_right_select;
+		int tooltip_len = strlen(tooltip);
 		int server_color = get_server_color(server_index);
 
 		if (server_color != -1)
@@ -1015,7 +1020,7 @@ void update_channel_border(int server_index)
 			f1_y = -1;
 		}
 
-		kr_x = chat_window_border -> ncols - (key_right_select_len + 2);
+		kr_x = chat_window_border -> ncols - (tooltip_len + 2);
 		if (kr_x < 0)
 			kr_y = -1;
 
@@ -1023,7 +1028,7 @@ void update_channel_border(int server_index)
 			mvwprintw(chat_window_border -> win, f1_y, f1_x, f1_help);
 
 		if (kr_y >= 0)
-			mvwprintw(chat_window_border -> win, kr_y, kr_x, key_right_select);
+			mvwprintw(chat_window_border -> win, kr_y, kr_x, tooltip);
 
 		if (server_color != -1)
 			color_off(chat_window_border, server_color);
