@@ -1,5 +1,5 @@
 /* GPLv2 applies
- * SVN revision: $Revision: 777 $
+ * SVN revision: $Revision: 861 $
  * (C) 2006-2014 by folkert@vanheusden.com
  */
 #define _GNU_SOURCE
@@ -677,7 +677,7 @@ int main(int argc, char *argv[])
 		{
 			if (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0)
 			{
-				popup_notify(TRUE, "The only command line parameter accepted is the file name of the\nconfiguration file. Leave empty to create a default one. Servers\ncan be configured from within f-irc (press F3).");
+				commandline_help();
 
 				exit_fi();
 			}
@@ -823,10 +823,6 @@ int main(int argc, char *argv[])
 
 			free(err_msg);
 		}
-		else if (c == KEY_F(3))		/* F3: add server */
-		{
-			add_server_menu();
-		}
 		else if (c == KEY_F(4))				/* F4 */
 		{
 			set_cursor_mode(CM_EDIT);
@@ -1077,7 +1073,7 @@ int main(int argc, char *argv[])
 			{
 				BOOL do_line = TRUE, do_commands = TRUE;
 
-				if (utf8_ascii_get_at(cur_channel() -> input, 0) == '@')
+				if (utf8_ascii_get_at(cur_channel() -> input, 0) == '@' && utf8_strlen(cur_channel() -> input) >= 2)
 				{
 					if (utf8_ascii_get_at(cur_channel() -> input, 1) == '/' || utf8_ascii_get_at(cur_channel() -> input, 1) == '@')
 					{
@@ -1202,7 +1198,14 @@ int main(int argc, char *argv[])
 			editline_redraw = TRUE;
 		}
 		else if (c == 6) /* ^F scrollback channel */
+		{
 			show_channel_history();
+
+			/* because of ^B functionality: */
+			reposition_editline_cursor();
+
+			editline_redraw = TRUE;
+		}
 		else if (c == 22)	/* ^V escape char */
 			add_escape_char();
 		else if (c == 25)	/* ^Y with-data only toggle */
