@@ -1,5 +1,5 @@
 /* GPLv2 applies
- * SVN revision: $Revision: 769 $
+ * SVN revision: $Revision: 863 $
  * (C) 2006-2014 by folkert@vanheusden.com
  */
 #define _GNU_SOURCE
@@ -47,9 +47,7 @@ void free_server(int server_index)
 
 	free_resolve_info(&pc -> ri);
 
-	for(loop=0; loop<pc -> n_all_channels; loop++)
-		myfree(pc -> all_channels[loop]);
-	myfree(pc -> all_channels);
+	free_channel_list(server_index);
 
 	for(loop=0; loop<pc -> n_channels; loop++)
 		free_channel(&pc -> pchannels[loop]);
@@ -168,9 +166,9 @@ int add_server(const char *host_and_port_in, const char *username, const char *p
 	server_list[n_servers].nickname = strdup(nickname);
 	server_list[n_servers].user_complete_name = strdup(complete_name ? complete_name : "");
 
-	server_list[n_servers].all_channels_complete = FALSE;
-	server_list[n_servers].all_channels = NULL;
-	server_list[n_servers].n_all_channels = 0;
+	server_list[n_servers].channel_list_complete = FALSE;
+	server_list[n_servers].channel_list = NULL;
+	server_list[n_servers].channel_list_n = 0;
 
 	init_string_array(&server_list[n_servers].auto_join);
 
@@ -239,6 +237,13 @@ conn_state_t get_state(int server_index)
 	return server_list[server_index].state;
 }
 
+long int get_state_age(int server_index)
+{
+	long int age = time(NULL) - server_list[server_index].state_since;
+
+	return age;
+}
+
 int get_server_color(int server_index)
 {
 	conn_state_t state = get_state(server_index);
@@ -262,7 +267,7 @@ void restart_server(int sr)
 	server_list[sr].state = STATE_NO_CONNECTION;
 	server_list[sr].state_since = 0;
 
-	server_list[sr].all_channels_complete = FALSE;
+	server_list[sr].channel_list_complete = FALSE;
 }
 
 void try_next_server(int sr)
@@ -277,7 +282,6 @@ void try_next_server(int sr)
 int register_server_events(struct pollfd **pfd, int *n_fd)
 {
 	int loop = 0, redraw_rc = 0;
-	time_t now = time(NULL);
 
 	for(loop=0; loop<n_servers; loop++)
 	{
@@ -285,18 +289,19 @@ int register_server_events(struct pollfd **pfd, int *n_fd)
 
 		if (get_state(loop) == STATE_NO_CONNECTION)
 		{
-			if (server_list[loop].state_since == 0 || now - server_list[loop].state_since >= delay_before_reconnect)
+			if (get_state_age(loop) >= delay_before_reconnect)
 			{
 				char *message = NULL;
 
 				update_statusline(loop, 0, "Resolving %s", server_list[loop].server_host);
 				mydoupdate();
 
-				if (resolve(server_list[loop].server_host, server_list[loop].server_port, &server_list[loop].ri, &message))
+				if (resolve(server_list[loop].server_host, server_list[loop].server_port, &server_list[loop].ri, &message) && server_list[loop].ri.alist_n > 0)
 					set_state(loop, STATE_TCP_CONNECT);
 				else
 				{
 					update_statusline(loop, 0, "Problem resolving %s: %s", server_list[loop].server_host, message);
+					set_state(loop, STATE_ERROR);
 
 					free(message);
 				}
@@ -379,7 +384,7 @@ int register_server_events(struct pollfd **pfd, int *n_fd)
 		}
 		else if (get_state(loop) == STATE_CONNECTED2)
 		{
-			if (time(NULL) - server_list[loop].state_since >= nick_sleep)
+			if (get_state_age(loop) >= nick_sleep)
 			{
 				if (irc_login2(&server_list[loop]) == -1)
 				{
@@ -682,3 +687,52 @@ void server_set_additional_nick(int sr, const char *n2)
 
 	server_list[sr].nickname2 = strdup(n2);
 }
+
+int compare_channel_list_item(const void *a, const void *b)
+{
+	channel_topic_t *pa = (channel_topic_t *)a;
+	channel_topic_t *pb = (channel_topic_t *)b;
+
+	return strcasecmp(pa -> channel, pb -> channel);
+}
+
+void free_channel_list(int sr)
+{
+	server *ps = &server_list[sr];
+	int index = 0;
+
+	for(index=0; index<ps -> channel_list_n; index++)
+	{
+		myfree(ps -> channel_list[index].channel);
+		myfree(ps -> channel_list[index].topic);
+	}
+
+	myfree(ps -> channel_list);
+
+	ps -> channel_list_n = 0;
+	ps -> channel_list = NULL;
+
+	ps -> channel_list_complete = FALSE;
+}
+
+int compare_channel_for_sort(const void *a, const void *b)
+{
+	channel *pa = (channel *)a;
+	channel *pb = (channel *)b;
+
+	return strcasecmp(pa -> channel_name, pb -> channel_name);
+}
+
+void sort_channels(int sr)
+{
+	server *ps = &server_list[sr];
+
+	/* server channel must always be at position 0 */
+	if (ps -> n_channels > 1)
+		qsort(&ps -> pchannels[1], ps -> n_channels - 1, sizeof(channel), compare_channel_for_sort);
+}
+
+server *gsr(int sr)
+{
+	return &server_list[sr];
+}
