@@ -1,5 +1,5 @@
 /* GPLv2 applies
- * SVN revision: $Revision: 771 $
+ * SVN revision: $Revision: 863 $
  * (C) 2006-2014 by folkert@vanheusden.com
  */
 #define _GNU_SOURCE
@@ -35,6 +35,26 @@
 #include "help.h"
 #include "scrollback.h"
 
+void keypress_visual_feedback(void)
+{
+	NEWWIN *popup = create_window(5, 25);
+
+	box(popup -> win, 0, 0);
+	mywattron(popup -> win, A_BOLD);
+	mvwprintw(popup -> win, 0, 2, "Executing command");
+	mywattroff(popup -> win, A_BOLD);
+
+	escape_print_xy(popup, 2, 2, "Please wait...");
+
+	mydoupdate();
+
+	usleep(100000);
+
+	delete_window(popup);
+
+	mydoupdate();
+}
+
 void popup_notify(BOOL use_getch, const char *format, ...)
 {
 	va_list ap;
@@ -364,7 +384,7 @@ int ping_user(int sr, const char *nick)
 	return log_channel(sr, 0, nick, "send ping", TRUE);
 }
 
-int dcc_send_user(int server_index, const char *nick)
+int dcc_send_user(int sr, int ch, const char *nick)
 {
 	int rc = 0;
 	const char *filename_in = edit_box(60, "^DCC send file^", NULL);
@@ -374,11 +394,11 @@ int dcc_send_user(int server_index, const char *nick)
 		char *log_msg = NULL;
 		const char *filename = explode_path(filename_in);
 
-		rc = init_send_dcc(filename, server_index, nick);
+		rc = init_send_dcc(filename, sr, ch, nick);
 
 		asprintf(&log_msg, "DCC send %s", filename);
 
-		if (rc == 0 && log_channel(server_index, 0, nick, log_msg, TRUE) == -1)
+		if (rc == 0 && log_channel(sr, ch, nick, log_msg, TRUE) == -1)
 			rc = -1;
 
 		myfree(filename);
@@ -391,7 +411,7 @@ int dcc_send_user(int server_index, const char *nick)
 	return rc;
 }
 
-int ctcp_user(int server_index, const char *nick)
+int ctcp_user(int sr, const char *nick)
 {
 	int rc = 0;
 	const char *ctcp_command = edit_box(60, "^CTCP^", NULL);
@@ -402,7 +422,7 @@ int ctcp_user(int server_index, const char *nick)
 
 		asprintf(&msg, "\001%s\001", ctcp_command);
 
-		rc = irc_privmsg(server_list[server_index].fd, nick, msg);
+		rc = irc_privmsg(server_list[sr].fd, nick, msg);
 
 		myfree(msg);
 
@@ -410,7 +430,7 @@ int ctcp_user(int server_index, const char *nick)
 
 		myfree(ctcp_command);
 
-		if (rc == 0 && log_channel(server_index, 0, nick, log_msg, TRUE) == -1)
+		if (rc == 0 && log_channel(sr, 0, nick, log_msg, TRUE) == -1)
 			rc = -1;
 
 		myfree(log_msg);
@@ -421,7 +441,7 @@ int ctcp_user(int server_index, const char *nick)
 
 void cmd_LEAVE(int server_nr, int server_channel_nr, const char *channel_name)
 {
-	int channel_index = -1;
+	int ch = -1;
 
 	/* see if there's a channel given */
 	if (channel_name && strlen(channel_name) > 0)
@@ -432,12 +452,12 @@ void cmd_LEAVE(int server_nr, int server_channel_nr, const char *channel_name)
 		{
 			if (strcasecmp(server_list[server_nr].pchannels[loop].channel_name, channel_name) == 0)
 			{
-				channel_index = loop;
+				ch = loop;
 				break;
 			}
 		}
 
-		if (channel_index == -1)
+		if (ch == -1)
 		{
 			update_statusline(server_nr, server_channel_nr, "Not in channel %s", channel_name);
 			wrong_key();
@@ -453,17 +473,17 @@ void cmd_LEAVE(int server_nr, int server_channel_nr, const char *channel_name)
 		}
 		else
 		{
-			channel_index = server_channel_nr;
+			ch = server_channel_nr;
 		}
 	}
 
-	if (channel_index != -1)
+	if (ch != -1)
 	{
 		int cur_channel = server_channel_nr;
 
-		update_statusline(server_nr, 0, "Channel %s closed", server_list[server_nr].pchannels[channel_index].channel_name);
+		update_statusline(server_nr, 0, "Channel %s closed", server_list[server_nr].pchannels[ch].channel_name);
 
-		close_channel(server_nr, channel_index, TRUE);
+		close_channel(server_nr, ch, TRUE);
 
 		if (server_nr == current_server)
 		{
@@ -541,7 +561,7 @@ int user_command(int current_server, int current_server_channel_nr, const char *
 				popup_notify(FALSE, "/ADDSERVER: missing host(-name) to connect to");
 			else
 			{
-				int server_index = add_server(pars,			/* host[:port] */
+				int sr = add_server(pars,			/* host[:port] */
 						server_list[current_server].username,	/* username */
 						NULL,					/* password */
 						server_list[current_server].nickname,	/* nickname */
@@ -549,9 +569,9 @@ int user_command(int current_server, int current_server_channel_nr, const char *
 						NULL					/* description */
 					  );
 
-				update_statusline(current_server, current_server_channel_nr, "Added server %s:%d", server_list[server_index].server_host, server_list[server_index].server_port);
+				update_statusline(current_server, current_server_channel_nr, "Added server %s:%d", server_list[sr].server_host, server_list[sr].server_port);
 
-				change_channel(server_index, 0, TRUE, TRUE);
+				change_channel(sr, 0, TRUE, TRUE);
 			}
 		}
 		else if (strcasecmp(command, "TIME") == 0)
@@ -658,7 +678,7 @@ int user_command(int current_server, int current_server_channel_nr, const char *
 					popup_notify(FALSE, "DCCSEND: nick '%s' is not known for this channel", pars);
 				else
 				{
-					if (init_send_dcc(end_of_nick + 1, current_server, pars) != 0)
+					if (init_send_dcc(end_of_nick + 1, current_server, current_server_channel_nr, pars) != 0)
 						popup_notify(FALSE, "DCCSEND: failed to start transmitting file");
 					else
 						update_statusline(current_server, current_server_channel_nr, "DCCSEND: waiting for user '%s' to acknowledge file '%s'", pars, end_of_nick + 1);
@@ -749,25 +769,25 @@ int user_command(int current_server, int current_server_channel_nr, const char *
 		}
 		else if (strcasecmp(command, "QUIT") == 0)	/* leave server */
 		{
-			int server_index = -1;
+			int sr = -1;
 
 			/* see if there's a channel given */
 			if (pars && strlen(pars) > 0)
 			{
 				int loop;
 
-				/* find server into server_index */
+				/* find server into sr */
 				for(loop=0; loop<n_servers; loop++)
 				{
 					if (strcasecmp(server_list[loop].server_host, pars) == 0)
 					{
-						server_index = loop;
+						sr = loop;
 						break;
 					}
 				}
 			}
 
-			if (server_index != -1)
+			if (sr != -1)
 			{
 				if (n_servers == 1)
 				{
@@ -779,11 +799,11 @@ int user_command(int current_server, int current_server_channel_nr, const char *
 				}
 				else
 				{
-					close_server(server_index, TRUE);
-					free_server(server_index);
+					close_server(sr, TRUE);
+					free_server(sr);
 
-					if (server_index <= (n_servers - 1))
-						memmove(&server_list[server_index], &server_list[server_index + 1], sizeof(server) * (n_servers - (server_index + 1)));
+					if (sr <= (n_servers - 1))
+						memmove(&server_list[sr], &server_list[sr + 1], sizeof(server) * (n_servers - (sr + 1)));
 
 					change_channel(0, 0, TRUE, FALSE);
 
@@ -912,16 +932,15 @@ int user_command(int current_server, int current_server_channel_nr, const char *
 	return rc;
 }
 
-int user_menu(int server_index, int channel_index, int name_index)
+int user_menu(int sr, int ch, int name_index)
 {
 	int rc = 0;
 	NEWWIN *bwin = NULL, *win = NULL;
-	channel *pc = &server_list[server_index].pchannels[channel_index];
-	char *channel_name = pc -> channel_name;
-	const char *nick = NULL, *name_complete = NULL, *user_host = NULL;
+	char *channel_name = gch(sr, ch) -> channel_name;
+	const char *nick = NULL;
 	char *title = NULL;
 
-	if (name_index < 0 || name_index > pc -> n_names)
+	if (name_index < 0 || name_index > gch(sr, ch) -> n_names)
 	{
 		delete_window(win);
 		delete_window(bwin);
@@ -930,21 +949,17 @@ int user_menu(int server_index, int channel_index, int name_index)
 		return 0;
 	}
 
-	nick = pc -> persons[name_index].nick;
-	name_complete = pc -> persons[name_index].complete_name;
-	user_host = pc -> persons[name_index].user_host;
+	nick = gch(sr, ch) -> persons[name_index].nick;
 
-	asprintf(&title, "%s [%s] (%s)", nick, channel_name, server_list[server_index].server_host);
+	if (gch(sr, ch) -> persons[name_index].complete_name == NULL)
+		(void)irc_whois(server_list[sr].fd, nick);
 
-	create_win_border(48, 18, title, &bwin, &win, FALSE);
+	asprintf(&title, "%s [%s] (%s)", nick, channel_name, server_list[sr].server_host);
+
+	create_win_border(60, 18, title, &bwin, &win, FALSE);
 
 	free(title);
 
-	mywattron(win -> win, A_BOLD);
-	mvwprintw(win -> win, 1, 2, "%s", name_complete);
-	mvwprintw(win -> win, 2, 2, "%s", user_host);
-	mywattroff(win -> win, A_BOLD);
-
 	escape_print_xy(win, 3, 2, "^k^ kick user, ^K^ kick with message");
 	escape_print_xy(win, 4, 2, "^b^ ban nick,  ^B^ ban+kick nick");
 	escape_print_xy(win, 5, 2, "^g^ ban user,  ^G^ ban+kick user");
@@ -959,20 +974,31 @@ int user_menu(int server_index, int channel_index, int name_index)
 	escape_print_xy(win, 14, 2, "^c^ CTCP");
 	escape_print_xy(win, 15, 2, "^q^/^LEFT cursor key^ exit this menu");
 
-	if (pc -> persons[name_index].ignored)
+	if (gch(sr, ch) -> persons[name_index].ignored)
 		escape_print_xy(win, 16, 2, "currently ignoring user");
 
 	for(;;)
 	{
 		int c = -1;
 
+		const char *name_complete = gch(sr, ch) -> persons[name_index].complete_name;
+		const char *user_host = gch(sr, ch) -> persons[name_index].user_host;
+
+		mywattron(win -> win, A_BOLD);
+		mvwprintw(win -> win, 1, 2, "%-44s", name_complete ? name_complete : "(name not yet known)");
+		mvwprintw(win -> win, 2, 2, "%-44s", user_host ? user_host : "(user & host not yet known)");
+		mywattroff(win -> win, A_BOLD);
+
 		mydoupdate();
 
-		c = wait_for_keypress(FALSE);
+		c = wait_for_keypress(TRUE);
 
 		if (c == KEY_LEFT || (c == KEY_MOUSE && right_mouse_button_clicked()))
 			break;
 
+		if (c <= 0)
+			continue;
+
 		if (c == 'q')
 			break;
 		else if (c == 7)
@@ -981,33 +1007,27 @@ int user_menu(int server_index, int channel_index, int name_index)
 			exit_fi();
 		else if (c == 'i')
 		{
-			channel *pc = &server_list[server_index].pchannels[channel_index];
+			gch(sr, ch) -> persons[name_index].ignored = TRUE;
 
-			pc -> persons[name_index].ignored = TRUE;
-
-			add_ignore(pc -> channel_name, pc -> persons[name_index].nick, pc -> persons[name_index].user_host);
+			add_ignore(gch(sr, ch) -> channel_name, gch(sr, ch) -> persons[name_index].nick, gch(sr, ch) -> persons[name_index].user_host);
 
 			if (!save_ignore_list())
 				popup_notify(FALSE, "Problem saving ignore-list file");
 
-			if (rc == 0 && log_channel(server_index, channel_index, nick, "ignore", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, ch, nick, "ignore", TRUE) == -1)
 				rc = -1;
-			break;
 		}
 		else if (c == 'I')
 		{
-			channel *pc = &server_list[server_index].pchannels[channel_index];
+			gch(sr, ch) -> persons[name_index].ignored = FALSE;
 
-			pc -> persons[name_index].ignored = FALSE;
-
-			del_ignore(pc -> channel_name, pc -> persons[name_index].nick);
+			del_ignore(gch(sr, ch) -> channel_name, gch(sr, ch) -> persons[name_index].nick);
 
 			if (!save_ignore_list())
 				popup_notify(FALSE, "Problem saving ignore-list file");
 
-			if (rc == 0 && log_channel(server_index, channel_index, nick, "unignore", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, ch, nick, "unignore", TRUE) == -1)
 				rc = -1;
-			break;
 		}
 		else if (c == 'k' || c == 'K')
 		{
@@ -1024,9 +1044,9 @@ int user_menu(int server_index, int channel_index, int name_index)
 
 			if (dokick)
 			{
-				rc = irc_kick(server_list[server_index].fd, channel_name, nick, msg);
+				rc = irc_kick(server_list[sr].fd, channel_name, nick, msg);
 
-				if (rc == 0 && log_channel(server_index, channel_index, nick, "kick", TRUE) == -1)
+				if (rc == 0 && log_channel(sr, ch, nick, "kick", TRUE) == -1)
 					rc = -1;
 			}
 
@@ -1038,9 +1058,9 @@ int user_menu(int server_index, int channel_index, int name_index)
 		{
 			const char *what = c == 'b' ? nick : user_host;
 
-			rc = irc_ban(server_list[server_index].fd, channel_name, what);
+			rc = irc_ban(server_list[sr].fd, channel_name, what);
 
-			if (rc == 0 && log_channel(server_index, channel_index, what, "BAN", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, ch, what, "BAN", TRUE) == -1)
 				rc = -1;
 			break;
 		}
@@ -1052,15 +1072,15 @@ int user_menu(int server_index, int channel_index, int name_index)
 			{
 				const char *what = c == 'B' ? nick : user_host;
 
-				rc = irc_ban(server_list[server_index].fd, channel_name, what);
+				rc = irc_ban(server_list[sr].fd, channel_name, what);
 
 				if (rc == -1)
-					update_statusline(server_index, channel_index, "BAN failed: also not kicked\n");
+					update_statusline(sr, ch, "BAN failed: also not kicked\n");
 				else
 				{
-					rc = irc_kick(server_list[server_index].fd, channel_name, what, msg);
+					rc = irc_kick(server_list[sr].fd, channel_name, what, msg);
 
-					if (rc == 0 && log_channel(server_index, channel_index, what, "BAN + KICK", TRUE) == -1)
+					if (rc == 0 && log_channel(sr, ch, what, "BAN + KICK", TRUE) == -1)
 						rc = -1;
 				}
 
@@ -1071,73 +1091,73 @@ int user_menu(int server_index, int channel_index, int name_index)
 		}
 		else if (c == 'o')
 		{
-			rc = irc_op(server_list[server_index].fd, channel_name, nick, TRUE);
+			rc = irc_op(server_list[sr].fd, channel_name, nick, TRUE);
 
-			if (rc == 0 && log_channel(server_index, channel_index, nick, "set OP", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, ch, nick, "set OP", TRUE) == -1)
 				rc = -1;
-			break;
 		}
 		else if (c == 'O')
 		{
-			rc = irc_op(server_list[server_index].fd, channel_name, nick, FALSE);
+			rc = irc_op(server_list[sr].fd, channel_name, nick, FALSE);
 
-			if (rc == 0 && log_channel(server_index, channel_index, nick, "remove OP", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, ch, nick, "remove OP", TRUE) == -1)
 				rc = -1;
-			break;
 		}
 		else if (c == 'w')
 		{
-			rc = irc_whois(server_list[server_index].fd, nick);
+			rc = irc_whois(server_list[sr].fd, nick);
 
-			if (rc == 0 && log_channel(server_index, 0, nick, "CTCP WHOIS", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, 0, nick, "WHOIS", TRUE) == -1)
 				rc = -1;
-			break;
 		}
 		else if (c == 'v')
 		{
-			rc = irc_privmsg(server_list[server_index].fd, nick, "\001VERSION\001");
+			rc = irc_privmsg(server_list[sr].fd, nick, "\001VERSION\001");
 
-			if (rc == 0 && log_channel(server_index, 0, nick, "CTCP VERSION", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, 0, nick, "CTCP VERSION", TRUE) == -1)
 				rc = -1;
-			break;
 		}
 		else if (c == 'a')
 		{
 			popup_notify(FALSE, "Speak allowed");
 
-			rc = irc_allowspeak(server_list[server_index].fd, channel_name, nick, TRUE);
+			rc = irc_allowspeak(server_list[sr].fd, channel_name, nick, TRUE);
 
-			if (rc == 0 && log_channel(server_index, channel_index, nick, "allow speak", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, ch, nick, "allow speak", TRUE) == -1)
 				rc = -1;
 		}
 		else if (c == 'A')
 		{
 			popup_notify(FALSE, "Speak disabled");
 
-			rc = irc_allowspeak(server_list[server_index].fd, channel_name, nick, FALSE);
+			rc = irc_allowspeak(server_list[sr].fd, channel_name, nick, FALSE);
 
-			if (rc == 0 && log_channel(server_index, channel_index, nick, "disallow speak", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, ch, nick, "disallow speak", TRUE) == -1)
 				rc = -1;
 		}
 		else if (c == 'p')
 		{
-			if (ping_user(server_index, nick) == -1)
+			if (ping_user(sr, nick) == -1)
 				rc = -1;
 			else
 				popup_notify(FALSE, "Ping sent");
 		}
 		else if (c == 'D')
 		{
-			if (dcc_send_user(server_index, nick) == -1)
+			if (dcc_send_user(sr, ch, nick) == -1)
 				rc = -1;
 		}
 		else if (c == 'c')
 		{
-			if (ctcp_user(server_index, nick) == -1)
+			if (ctcp_user(sr, nick) == -1)
 				rc = -1;
 		}
 		else
+		{
 			wrong_key();
+		}
+
+		keypress_visual_feedback();
 	}
 
 	delete_window(win);
@@ -1287,24 +1307,22 @@ void edit_headline_matchers()
 
 void edit_send_after_connect(int sr)
 {
-	server *ps = &server_list[sr];
 	char *title = NULL;
 
-	asprintf(&title, "EDIT %s \"send after connect\", left cursor to exit", ps -> server_host);
+	asprintf(&title, "EDIT %s \"send after connect\", left cursor to exit", gsr(sr) -> server_host);
 
-	edit_string_array(&ps -> send_after_login, title);
+	edit_string_array(&gsr(sr) -> send_after_login, title);
 
 	free(title);
 }
 
 void edit_auto_join(int sr)
 {
-	server *ps = &server_list[sr];
 	char *title = NULL;
 
-	asprintf(&title, "EDIT %s \"auto-join\", left cursor to exit", ps -> server_host);
+	asprintf(&title, "EDIT %s \"auto-join\", left cursor to exit", gsr(sr) -> server_host);
 
-	edit_string_array(&ps -> auto_join, title);
+	edit_string_array(&gsr(sr) -> auto_join, title);
 
 	free(title);
 }
@@ -1550,11 +1568,10 @@ BOOL edit_string_array(string_array_t *p, const char *title)
 void edit_server(int sr)
 {
 	NEWWIN *bwin = NULL, *win = NULL;
-	server *ps = &server_list[sr];
 	char *title = NULL;
 	BOOL restart_required = FALSE;
 
-	asprintf(&title, "EDIT %s:%d, left cursor key to exit", ps -> server_host, ps -> server_port);
+	asprintf(&title, "EDIT %s:%d, left cursor key to exit", gsr(sr) -> server_host, gsr(sr) -> server_port);
 
 	create_win_border(63, 16, title, &bwin, &win, FALSE);
 
@@ -1597,22 +1614,22 @@ void edit_server(int sr)
 		}
 		else if (c == 'd')
 		{
-			const char *n = edit_box(60, "^Edit server description^", ps -> description);
+			const char *n = edit_box(60, "^Edit server description^", gsr(sr) -> description);
 
 			if (n != NULL)
 			{
-				myfree(ps -> description);
-				ps -> description = n;
+				myfree(gsr(sr) -> description);
+				gsr(sr) -> description = n;
 			}
 		}
 		else if (c == 'h')
 		{
-			const char *n = edit_box(60, "^Edit server host^", ps -> server_host);
+			const char *n = edit_box(60, "^Edit server host^", gsr(sr) -> server_host);
 
 			if (n != NULL)
 			{
-				myfree(ps -> server_host);
-				ps -> server_host = n;
+				myfree(gsr(sr) -> server_host);
+				gsr(sr) -> server_host = n;
 			}
 
 			restart_required = TRUE;
@@ -1622,59 +1639,59 @@ void edit_server(int sr)
 			char *port_str = NULL;
 			const char *n = NULL;
 
-			asprintf(&port_str, "%d", ps -> server_port);
+			asprintf(&port_str, "%d", gsr(sr) -> server_port);
 			n = edit_box(60, "^Edit server port^", port_str);
 			free(port_str);
 
 			if (n != NULL)
-				ps -> server_port = atoi(n);
+				gsr(sr) -> server_port = atoi(n);
 
 			restart_required = TRUE;
 		}
 		else if (c == 'U')
 		{
-			char *n = (char *)edit_box(60, "^Edit \"complete name\"^", ps -> user_complete_name);
+			char *n = (char *)edit_box(60, "^Edit \"complete name\"^", gsr(sr) -> user_complete_name);
 
 			if (n != NULL)
 			{
-				free(ps -> user_complete_name);
-				ps -> user_complete_name = n;
+				free(gsr(sr) -> user_complete_name);
+				gsr(sr) -> user_complete_name = n;
 			}
 
 			restart_required = TRUE;
 		}
 		else if (c == 'u')
 		{
-			char *n = (char *)edit_box(60, "^Edit user name^", ps -> username);
+			char *n = (char *)edit_box(60, "^Edit user name^", gsr(sr) -> username);
 
 			if (n != NULL)
 			{
-				free(ps -> username);
-				ps -> username = n;
+				free(gsr(sr) -> username);
+				gsr(sr) -> username = n;
 			}
 
 			restart_required = TRUE;
 		}
 		else if (c == 'P')
 		{
-			char *n = (char *)edit_box(60, "^Edit password^", ps -> password);
+			char *n = (char *)edit_box(60, "^Edit password^", gsr(sr) -> password);
 
 			if (n != NULL)
 			{
-				free(ps -> password);
-				ps -> password = n;
+				free(gsr(sr) -> password);
+				gsr(sr) -> password = n;
 			}
 
 			restart_required = TRUE;
 		}
 		else if (c == 'n')
 		{
-			char *n = (char *)edit_box(60, "^Edit nick^", ps -> nickname);
+			char *n = (char *)edit_box(60, "^Edit nick^", gsr(sr) -> nickname);
 
 			if (n != NULL)
 			{
-				free(ps -> nickname);
-				ps -> nickname = n;
+				free(gsr(sr) -> nickname);
+				gsr(sr) -> nickname = n;
 			}
 
 			restart_required = TRUE;
@@ -1701,10 +1718,9 @@ void edit_server(int sr)
 void server_menu(int sr)
 {
 	NEWWIN *bwin = NULL, *win = NULL;
-	server *ps = &server_list[sr];
 	char *title = NULL;
 
-	asprintf(&title, "%s [%s]", ps -> description, ps -> server_host);
+	asprintf(&title, "%s [%s]", gsr(sr) -> description, gsr(sr) -> server_host);
 
 	create_win_border(63, 16, title, &bwin, &win, FALSE);
 
@@ -1718,24 +1734,24 @@ void server_menu(int sr)
 		char *addr = NULL, *addr_real = NULL, *bps_str = NULL;
 		time_t now = time(NULL);
 		double bps = 0.0;
-		int t_diff = now - ps -> ts_bytes;
+		int t_diff = now - gsr(sr) -> ts_bytes;
 
 		werase(win -> win);
 
-		asprintf(&addr, "address: ^[%s]:%d^", ps -> server_host, ps -> server_port);
+		asprintf(&addr, "address: ^[%s]:%d^", gsr(sr) -> server_host, gsr(sr) -> server_port);
 		escape_print_xy(win, 3, 2, addr);
 		free(addr);
 
-		asprintf(&addr_real, "connected to: ^%s^", ps -> server_real);
+		asprintf(&addr_real, "connected to: ^%s^", gsr(sr) -> server_real);
 		escape_print_xy(win, 4, 2, addr_real);
 		free(addr_real);
 
-		bps = ps -> prev_bps;
+		bps = gsr(sr) -> prev_bps;
 
 		if (t_diff)
-			bps = (bps + ps -> bytes / (double)t_diff) / 2;
+			bps = (bps + gsr(sr) -> bytes / (double)t_diff) / 2;
 
-		asprintf(&bps_str, "BPS: %.2f  latency: %fs", bps, ps -> server_latency);
+		asprintf(&bps_str, "BPS: %.2f  latency: %fs", bps, gsr(sr) -> server_latency);
 
 		escape_print_xy(win, 5, 2, bps_str);
 		free(bps_str);
@@ -1747,16 +1763,16 @@ void server_menu(int sr)
 		escape_print_xy(win, 11, 2, "^e^ edit server settings");
 		escape_print_xy(win, 14, 2, "^q^/^LEFT cursor key^ exit this menu");
 
-		if (ps -> all_channels_complete == TRUE)
-			asprintf(&ch_list_compl, "List of channels available (%d known)", ps -> n_all_channels);
+		if (gsr(sr) -> channel_list_complete == TRUE)
+			asprintf(&ch_list_compl, "List of channels available (%d known)", gsr(sr) -> channel_list_n);
 		else
-			asprintf(&ch_list_compl, "List of channels NOT YET available (received: %d)", ps -> n_all_channels);
+			asprintf(&ch_list_compl, "List of channels NOT YET available (received: %d)", gsr(sr) -> channel_list_n);
 
 		escape_print_xy(win, 1, 2, ch_list_compl);
 
 		free(ch_list_compl);
 
-		switch(ps -> state)
+		switch(gsr(sr) -> state)
 		{
 			case STATE_NO_CONNECTION:
 				ss = "Server state: no connection, will connect";
@@ -1845,7 +1861,7 @@ void server_menu(int sr)
 		}
 		else if (c == 'g')
 		{
-			if (irc_list(ps -> fd) == -1)
+			if (irc_list(gsr(sr) -> fd) == -1)
 			{
 				popup_notify(FALSE, "Disconnected from server");
 				break;
@@ -1861,24 +1877,32 @@ void server_menu(int sr)
 		}
 		else if (c == 'j')
 		{
-			if (ps -> all_channels_complete == FALSE)
+			if (gsr(sr) -> channel_list_complete == FALSE)
 				popup_notify(FALSE, "Please wait for the channel list to load");
 			else
 			{
-				buffer *temp = create_buffer(server_list[sr].n_all_channels);
+				buffer *temp = create_buffer(server_list[sr].channel_list_n);
 				int loop, csr = -1, cch = -1;
 				char *dummy = NULL;
 				BOOL err = FALSE;
 				NEWWIN *bwin = NULL, *win = NULL;
 
-				create_win_border(40, max_y - 4, "right cursor key: select, left: cancel", &bwin, &win, FALSE);
+				create_win_border(max_x - 6, max_y - 4, "right cursor key: select, left: cancel", &bwin, &win, FALSE);
 
-				for(loop=0; loop<server_list[sr].n_all_channels; loop++)
-					add_to_buffer(temp, server_list[sr].all_channels[loop], NULL, FALSE, sr, loop);
+				for(loop=0; loop<server_list[sr].channel_list_n; loop++)
+				{
+					char *line = NULL;
+
+					asprintf(&line, "%-12s %s", gsr(sr) -> channel_list[loop].channel, gsr(sr) -> channel_list[loop].topic);
+
+					add_to_buffer(temp, line, NULL, FALSE, sr, loop);
+
+					free(line);
+				}
 
 				if (scrollback_and_select(win, temp, &dummy, &csr, &cch, server_list[sr].nickname, FALSE, FALSE))
 				{
-					if (irc_join(server_list[sr].fd, get_from_buffer(temp, cch) -> msg))
+					if (irc_join(server_list[sr].fd, gsr(sr) -> channel_list[cch].channel))
 						err = TRUE;
 				}
 
@@ -1923,15 +1947,15 @@ void add_server_menu(void)
 	myfree(nick);
 }
 
-int user_channel_menu(int server_index, const char *nick)
+int user_channel_menu(int sr, const char *nick)
 {
 	int rc = 0;
 	NEWWIN *bwin = NULL, *win = NULL;
 	char *title = NULL;
 
-	asprintf(&title, "%s (%s)", nick, server_list[server_index].server_host);
+	asprintf(&title, "%s (%s)", nick, server_list[sr].server_host);
 
-	create_win_border(40, 9, title, &bwin, &win, FALSE);
+	create_win_border(40, 9, title, &bwin, &win, TRUE);
 
 	free(title);
 
@@ -1957,37 +1981,39 @@ int user_channel_menu(int server_index, const char *nick)
 			break;
 		else if (c == 3)
 			exit_fi();
+		else if (c == KEY_F(1))
+			user_channel_menu_help();
 		else if (c == 'w')
 		{
-			rc = irc_whois(server_list[server_index].fd, nick);
+			rc = irc_whois(server_list[sr].fd, nick);
 
-			if (rc == 0 && log_channel(server_index, 0, nick, "CTCP WHOIS", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, 0, nick, "CTCP WHOIS", TRUE) == -1)
 				rc = -1;
 			break;
 		}
 		else if (c == 'v')
 		{
-			rc = irc_privmsg(server_list[server_index].fd, nick, "\001VERSION\001");
+			rc = irc_privmsg(server_list[sr].fd, nick, "\001VERSION\001");
 
-			if (rc == 0 && log_channel(server_index, 0, nick, "CTCP VERSION", TRUE) == -1)
+			if (rc == 0 && log_channel(sr, 0, nick, "CTCP VERSION", TRUE) == -1)
 				rc = -1;
 			break;
 		}
 		else if (c == 'p')
 		{
-			if (ping_user(server_index, nick) == -1)
+			if (ping_user(sr, nick) == -1)
 				rc = -1;
 			else
 				popup_notify(FALSE, "Ping sent");
 		}
 		else if (c == 'D')
 		{
-			if (dcc_send_user(server_index, nick) == -1)
+			if (dcc_send_user(sr, 0, nick) == -1)
 				rc = -1;
 		}
 		else if (c == 'c')
 		{
-			if (ctcp_user(server_index, nick) == -1)
+			if (ctcp_user(sr, nick) == -1)
 				rc = -1;
 		}
 		else
@@ -2250,7 +2276,7 @@ BOOL configure_firc(void)
 	NEWWIN *bwin = NULL, *win = NULL;
 	int cursor = 0, offset = 0;
 	BOOL redraw = TRUE;
-	const int win_height = 21, win_width = 68, content_height = win_height - 6, content_offset = 5;
+	const int win_height = 21, win_width = 68, content_offset = 6, content_height = win_height - (content_offset + 1);
 	int cnt = 0, cnf_len = 0, val_pos = 50;
 	const char title[] = " Configure f-irc, key right to select, key left to exit ";
 	const char *find = NULL;
@@ -2280,8 +2306,9 @@ BOOL configure_firc(void)
 			werase(win -> win);
 
 			escape_print_xy(win, 1, 1, "^F2^: store configuration on disk");
-			escape_print_xy(win, 2, 1, "^s^: edit server(s)  ^l^ edit favorites list  ^d^ edit dictionary");
-			escape_print_xy(win, 3, 1, "^m^: edit highlight matchers  ^h^ edit headline matchers");
+			escape_print_xy(win, 2, 1, "^a^: add server  ^s^: edit server(s)");
+			escape_print_xy(win, 3, 1, "^l^ edit favorites list  ^d^ edit dictionary");
+			escape_print_xy(win, 4, 1, "^m^: edit highlight matchers  ^h^ edit headline matchers");
 
 			for(loop=0; loop<min(max_rows, content_height); loop++)
 			{
@@ -2363,6 +2390,8 @@ BOOL configure_firc(void)
 			configure_firc_help();
 		else if (c == KEY_F(2))
 			save_config_with_popup();
+		else if (c == 'a')
+			add_server_menu();
 		else if (c == 's')
 		{
 			for(;;)
